var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _newArrowCheck(innerThis, boundThis) { if (innerThis !== boundThis) { throw new TypeError("Cannot instantiate an arrow function"); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var proauth = function () {
  'use strict';

  var _this3 = this;

  var SessionContainer = function () {

    /**
     * Create a session container.
     *
     * @param {String} namespace The namespace
     * @param {Storage} storage The session storage
     */
    function SessionContainer(namespace, storage) {
      _classCallCheck(this, SessionContainer);

      this.namespace = namespace;
      this.storage = storage;
    }

    SessionContainer.prototype.clear = function clear() {
      this.storage.removeItem(this.namespace);
    };

    _createClass(SessionContainer, [{
      key: "content",
      get: function get() {
        try {
          return JSON.parse(this.storage.getItem(this.namespace));
        } catch (error) {}
        return null;
      },
      set: function set(data) {
        if (data) {
          this.storage.setItem(this.namespace, JSON.stringify(data || null));
        } else {
          this.clear();
        }
      }
    }]);

    return SessionContainer;
  }();

  var OAuth2Response = function () {

    /**
     * Create an OAuth2.0 response.
     *
     * @param {Response} data The response
     */
    function OAuth2Response(data) {
      _classCallCheck(this, OAuth2Response);

      this.accessToken = data.access_token;
      this.refreshToken = data.refresh_token;
      this.expiresIn = data.expires_in;
      this.tokenType = data.token_type;
      this.scope = data.scope;
    }

    OAuth2Response.prototype.toObject = function toObject() {
      return Object.assign({}, this);
    };

    return OAuth2Response;
  }();

  var OAuth2Error = function () {

    /**
     * Create an OAuth2.0 error.
     *
     * @param {Response} response The response
     */
    function OAuth2Error(response) {
      _classCallCheck(this, OAuth2Error);

      this.response = response;
    }

    _createClass(OAuth2Error, [{
      key: "status",
      get: function get() {
        return this.response.status || null;
      }
    }, {
      key: "title",
      get: function get() {
        return this.response.title || null;
      }
    }, {
      key: "detail",
      get: function get() {
        return this.response.detail || null;
      }
    }]);

    return OAuth2Error;
  }();

  var OAuth2Client = function () {

    /**
     * Create an OAuth2.0 client instance.
     *
     * @param {string} apiUrl The url of OAuth2 endpoint
     * @param {string} clientId A string identifying the instance of the client
     * @param {Function} fetch The fetch function the client will use
     */
    function OAuth2Client(apiUrl, clientId, fetch) {
      var _this = this;

      _classCallCheck(this, OAuth2Client);

      /** @type {string} */
      this.apiUrl = apiUrl;

      /** @type {string} */
      this.clientId = clientId;

      this.fetch = function (init) {
        _newArrowCheck(this, _this);

        return fetch(this.apiUrl, init);
      }.bind(this);
    }

    OAuth2Client.prototype.handleAuthenticationResponse = function handleAuthenticationResponse(response) {
      var _this2 = this;

      if (response.ok) {
        // Authentication OK
        return response.json().then(function (j) {
          _newArrowCheck(this, _this2);

          return new OAuth2Response(j);
        }.bind(this));
      }

      // Authentication refused
      return response.json().then(function (j) {
        _newArrowCheck(this, _this2);

        return Promise.reject(new OAuth2Error(j));
      }.bind(this));
    };

    /**
     * Grant an access token.
     *
     * @param {Object} params An object containing the parameters to post to the OAuth2 endpoint
     * @return {Promise} A promise
     * @private
     */


    OAuth2Client.prototype.grantAccessToken = function grantAccessToken(params) {
      return this.fetch({
        method: "POST",
        headers: {
          "Content-Type": "application/json",
          "Accept": "application/json"
        },
        body: JSON.stringify(params)
      });
    };

    /**
     * Grant a token for an user.
     *
     * @param {String} username The user for which to request a token
     * @param {String} password The password to authenticate such user
     * @return {Promise} A promise
     */


    OAuth2Client.prototype.userCredentials = function userCredentials(username, password) {
      return this.grantAccessToken({
        "client_id": this.clientId,
        "grant_type": "password",
        username: username,
        password: password
      });
    };

    /**
     * Refresh an overdue token.
     *
     * @param {String} refreshToken The token to refresh
     * @return {Promise} A promise
     */


    OAuth2Client.prototype.refreshToken = function refreshToken(_refreshToken) {
      return this.grantAccessToken({
        "client_id": this.clientId,
        "grant_type": "refresh_token",
        "refresh_token": _refreshToken
      });
    };

    return OAuth2Client;
  }();

  var sendMessage = function (serviceWorker, message) {
    _newArrowCheck(this, _this3);

    // This wraps the message posting/response in a promise,
    // which will resolve if the response doesn't
    // contain an error, and reject with the error if it does.
    // If you'd prefer, it's possible to call
    // controller.postMessage() and set up the onmessage handler
    // independently of a promise, but this is a convenient wrapper.
    return new Promise(function (resolve, reject) {
      _newArrowCheck(this, _this3);

      var messageChannel = new MessageChannel();
      messageChannel.port1.onmessage = function (event) {
        _newArrowCheck(this, _this3);

        if (event.data.error) {
          reject(event.data.error);
        } else {
          resolve(event.data);
        }
      }.bind(this);

      // This sends the message data as well as transferring messageChannel.port2
      // to the service worker.
      // The service worker can then use the transferred port to reply via postMessage(),
      // which will in turn trigger the onmessage handler on messageChannel.port1.
      // See https://html.spec.whatwg.org/multipage/workers.html#dom-worker-postmessage.
      serviceWorker.controller.postMessage(message, [messageChannel.port2]);
    }.bind(this));
  }.bind(this);

  var Client = function () {

    /**
     * @typedef {Object} ClientSettings
     * @property {String} namespace
     * @property {String} oauthUrl
     * @property {String} oauthClientId
     * @property {String} sessionStorage
     */

    /**
     * Create a client.
     *
     * @param {ClientSettings} settings An object containing the client settings
     */
    function Client(settings) {
      var _this4 = this;

      _classCallCheck(this, Client);

      var namespace = settings.namespace,
          sessionStorage = settings.sessionStorage;

      this.settings = settings;
      this.sessionContainer = new SessionContainer(namespace, window[sessionStorage]);
      this._onMessage = function (event) {
        _newArrowCheck(this, _this4);

        var _event$data = event.data,
            broadcast = _event$data.broadcast,
            ns = _event$data.namespace,
            command = _event$data.command,
            params = _event$data.params;

        if (!broadcast || ns !== this.settings.namespace) {
          return;
        }

        switch (command) {
          case "session":
            this.sessionContainer.content = params[0];
            break;

          default:
        }

        // (todo): set session race condition must be avoided, dirty-checking data may be a solution
      }.bind(this);
    }

    Client.prototype.setSession = function setSession(data) {
      return sendMessage(this.serviceWorker, {
        namespace: this.settings.namespace,
        command: "setSession",
        params: [data]
      });
    };

    Client.prototype.clearSession = function clearSession() {
      return this.setSession(null);
    };

    Client.prototype.hasSession = function hasSession() {
      return !!this.sessionContainer.content;
    };

    Client.prototype.login = function login(user, pass) {
      var _this5 = this;

      if (!this.ready) {
        throw Error("proauth.js is not ready yet.");
      }
      var _settings = this.settings,
          oauthUrl = _settings.oauthUrl,
          oauthClientId = _settings.oauthClientId;

      return new OAuth2Client(oauthUrl, oauthClientId, window.fetch).userCredentials(user, pass).then(function (response) {
        _newArrowCheck(this, _this5);

        return response.ok ? response : Promise.reject(response);
      }.bind(this));
    };

    _createClass(Client, [{
      key: "ready",
      get: function get() {
        return this._serviceWorker && this._ready;
      }
    }, {
      key: "legacyMode",
      get: function get() {
        return this._legacyMode;
      },
      set: function set(sw) {
        throw Error("legacyMode is a read-only property");
      }
    }, {
      key: "serviceWorker",
      get: function get() {
        if (!this._serviceWorker) {
          throw Error("proauth.js: serviceWorker has been not registered yet.");
        }
        return this._serviceWorker;
      },
      set: function set(sw) {
        var _this6 = this;

        if (this._serviceWorker) {
          this._serviceWorker.removeEventListener("message", this._onMessage);
        }

        this._ready = false;
        this._serviceWorker = sw;
        this._legacyMode = sw !== navigator.serviceWorker;

        sendMessage(sw, {
          namespace: this.settings.namespace,
          command: "init",
          params: [this.settings, this.sessionContainer.content]
        }).then(function (data) {
          _newArrowCheck(this, _this6);

          this._serviceWorker.addEventListener("message", this._onMessage);
          this._ready = !!data;
          // (todo): dispatch global ready event
        }.bind(this));
      }
    }]);

    return Client;
  }();

  var absolutePath = function (href) {
    _newArrowCheck(this, _this3);

    var link = document.createElement("a");
    link.href = href;
    return String(link.protocol) + "//" + String(link.host) + String(link.pathname) + String(link.search) + String(link.hash);
  }.bind(this);

  var boot = function () {
    _newArrowCheck(this, _this3);

    var settings = {
      // Put defaults here
      swSrc: "/service-worker.js",
      swOptions: {},
      oauthUrl: "/oauth",
      oauthClientId: "proauth",
      sessionStorage: "localStorage",
      namespace: document.origin && document.origin !== "null" ? document.origin : "",
      managedUrls: ["/" // Manage the current url root by default
      ]
    };

    // Copy user settings
    Object.assign(settings, window.proauth || {});

    // Patch settings
    settings.oauthUrl = absolutePath(settings.oauthUrl);
    for (var k in settings.managedUrls) {
      settings.managedUrls[k] = absolutePath(settings.managedUrls[k]);
    }

    var client = new Client(settings);

    return client;
  }.bind(this);

  var proauthClient = {
    client: boot()
  };

  var client = proauthClient.client;

  if (navigator.serviceWorker) {

    if (client.settings.swSrc) {
      navigator.serviceWorker.register(client.settings.swSrc, client.settings.swOptions);
    }

    navigator.serviceWorker.ready.then(function (registration) {
      _newArrowCheck(this, _this3);

      // SW is ready, but its state can be either activating or activated.
      // We must wait until it's activated, so the clients will be claimed.

      if (registration.active.state === "activated") {

        client.serviceWorker = navigator.serviceWorker;
      } else {

        registration.active.addEventListener("statechange", function () {
          _newArrowCheck(this, _this3);

          if (registration.active.state === "activated") {

            client.serviceWorker = navigator.serviceWorker;
          }
        }.bind(this));
      }
    }.bind(this));
  }

  var _default = { client: client };

  return _default;
}();
//# sourceMappingURL=default.es2015.js.map
